
// Project: Communications 
// Created: 2015-04-25

#include "communications.agc"

// set window properties
SetWindowTitle( "Communications" )
SetWindowSize( 640, 480, 0 )

// set display properties
SetVirtualResolution( 1024, 768 )
SetOrientationAllowed( 1, 1, 1, 1 )

// Create 2 options - Host and Client
// In a simple demo like this, it is not necessary
// to track the image numbers. Normally, you would 
// assign traceable IDs to the images.

global glowTween as integer

global noteC as integer
global noteD as integer
global noteE as integer

global btnHost as integer
global btnClient as integer
global btnRed as integer
global btnGreen as integer
global btnBlue as integer
global btnNoteC as integer
global btnNoteD as integer
global btnNoteE as integer

btnHost = CreateSprite(LoadImage("\media\btnHost.jpg"))
btnClient = CreateSprite(LoadImage("\media\btnClient.jpg"))

// Load the program resources
noteC = LoadSound("\media\noteC.wav")
noteD = LoadSound("\media\noteD.wav")
noteE = LoadSound("\media\noteE.wav")

//Create the tween
glowTween = CreateTweenSprite(0.5)
SetTweenSpriteBlue(glowTween,128,255,5)
SetTweenSpriteRed(glowTween,128,255,5)
SetTweenSpriteGreen(glowTween,128,255,5)

// Wait for the user to enter Host or Client
	do
		SelectMode()
		
		// HOST Selected, create network and continue.
		if gNetwork.Status = cHOST
			success = CommsCreateHost()
			if success = 1
				exit
			else
				SelectMode()
			endif
		endif
		
		// CLIENT Selected, join network and continue.
		if gNetwork.Status = cCLIENT
			success = CommsJoinNetwork()
			if success = 1
				exit
			else
				selectMode()
			endif
		endif
		
	loop


	// Make the Main screem
	// 6 buttons for 3 colours and 3 notes
	btnRed = CreateSprite(LoadImage("\media\btnRed.jpg"))
	btnGreen = CreateSprite(LoadImage("\media\btnGreen.jpg"))
	btnBlue = CreateSprite(LoadImage("\media\btnBlue.jpg"))
	btnNoteC = CreateSprite(LoadImage("\media\btnNote.jpg"))
	btnNoteD = CreateSprite(LoadImage("\media\btnNote.jpg"))
	btnNoteE = CreateSprite(LoadImage("\media\btnNote.jpg"))

	SetSpritePositionByOffset(btnRed, getvirtualWidth() / 4, GetVirtualHeight() / 3 * 2)
	SetSpritePositionByOffset(btnGreen, getvirtualWidth() / 2, GetVirtualHeight() / 3 * 2)
	SetSpritePositionByOffset(btnBlue, getvirtualWidth() / 4 * 3, GetVirtualHeight() / 3 * 2)
	SetSpritePositionByOffset(btnNoteC, getvirtualWidth() / 4, GetVirtualHeight() / 3)
	SetSpritePositionByOffset(btnNoteD, getvirtualWidth() / 2, GetVirtualHeight() / 3)
	SetSpritePositionByOffset(btnNoteE, getvirtualWidth() / 4 * 3, GetVirtualHeight() / 3)

	do
		
		//Each cycle, check if a button was pressed
		checkButtonClicks()
		
		//Each cycle, check if there are any new messages
		checkMessagesReceived()
		
		updateAllTweens(GetFrameTime())
		Sync()
		
	loop

	end

//************************************************
function SelectMode()
//************************************************
// This function is not specific to the communications module.
// It simply displays 2 buttons to select the mode
// On selection it will set the gNetwork.status
// and then revert control back to the main program.
//************************************************
	
	SetSpritePositionByOffset(btnHost, (GetVirtualWidth() / 3), GetVirtualHeight() / 2)
	SetSpritePositionByOffset(btnClient, (GetVirtualWidth() / 3 * 2), GetVirtualHeight() / 2)

	
	do
		
		//Resize Button on click
		if GetPointerPressed() = 1
			hit = GetSpriteHit(getpointerX(), getpointerY())
			if hit = btnHost or hit = btnClient
				SetSpriteScaleByOffset(hit, 0.95,0.95)
			endif
		endif
		
		//Set Host/Client State on button release
		if GetPointerReleased() = 1
			hit = GetSpriteHit(getpointerX(), getpointerY())
			if hit = btnHost or hit = btnClient
				if hit = btnHost
					gNetwork.Status = cHOST
				else
					gNetwork.status = cCLIENT
				endif
				exit
			endif
		endif
		
		sync()
		
	loop

	SetSpritePositionByOffset(btnHost, -1000,0)
	SetSpritePositionByOffset(btnClient, -1000,0)
	
endfunction


//************************************************
function checkMessagesReceived()
//************************************************
//	Receive the messages and then process any received
//************************************************

	CommsReceiveMessage()
	// Check each message
	for n = 1 to CommsMessageCount()
		// Get the message type by retrieving the first token
		message$ = CommsGetMessage(n)
		msgType$ = GetStringToken(message$,":",1)
		// Process the message according to the token
		select msgType$
			
			//The other client has requested a note to be played
			case "NOTE"
				note$ = GetStringToken(message$,":",2)
				s = 0
				select note$
					case "C"
						s = PlaySound(noteC)
						PlayTweenSprite(glowTween,btnNoteC,0)
					endCase
					
					case "D"
						s = PlaySound(noteD)
						PlayTweenSprite(glowTween,btnNoteD,0)
					endCase
					
					case "E"
						s = PlaySound(noteE)
						PlayTweenSprite(glowTween,btnNoteE,0)
					endCase
					
				endselect
				
				if s > 0
					if CommsIsHost()
						SetSoundInstanceBalance(s,-1)
					else
						SetSoundInstanceBalance(s,1)
					endif
				endif
			endcase
			
			// The other client has requested a colour change
			case "COLOUR"
				colour$ = GetStringToken(message$,":",2)
				select colour$
					case "R"
						SetClearColor(255,128,128)
						ClearScreen()
						PlayTweenSprite(glowTween,btnRed,0)
					endcase
					case "G"
						SetClearColor(128,255,128)
						ClearScreen()
						PlayTweenSprite(glowTween,btnGreen,0)
					endcase
					case "B"
						SetClearColor(128,128,255)
						ClearScreen()
						PlayTweenSprite(glowTween,btnBlue,0)
					endcase
					
				endselect
			endcase
			
		endselect
		
	next n 
	
endfunction

//************************************************
function CheckButtonClicks()
//************************************************
//	Checks if any buttons were pressed
//	If they were, the appropriate message is constructed
//	and sent to the other clients/host
//************************************************

	if GetPointerReleased()
		hit = GetSpriteHit(getPointerX(), getPointerY())
		select hit
			case btnNoteC
				CommsSendMessage("NOTE:C")
			endcase

			case btnNoteD
				CommsSendMessage("NOTE:D")
			endcase

			case btnNoteE
				CommsSendMessage("NOTE:E")
			endcase
			
			case btnRed
				CommsSendMessage("COLOUR:R")
			endcase

			case btnGreen
				CommsSendMessage("COLOUR:G")
			endcase

			case btnBlue
				CommsSendMessage("COLOUR:B")
			endcase

		endselect
		
	endif

endfunction
